# XTENSION HELPERS
# v 1.0 5/14/2017 by james@sentman.com
# these are constants and helper functions used by XTension in its support
# for interface connections written in Python.
# do not alter this file.
#
# This file must be included with each python plugin that you create, this version
# is specifically for version 1 of the plugin API and should be included with any plugin
# that requires that API version.
#
# since this must be included with your own work there is no copyright or copying restrictions
# applied. Do with this as you wish. Free as in beer.

from XTension import *


# log coloring constants
# you can pass these as the second parameter to the writeLog command
# or instead you can pass a 6 character html style color value and the log
# line will be written in that color. "RRGGBB" 
# if no second parameter to writeLog is specified the log line will be blue.
# writeLog( "your message in blue by default") 
# writeLog( "an important message in red, xtColorRed)
# writeLog( "a message in orange", "FFCC44")
# using print( "a message") will equally result in a blue line to the XTension log

xtColorBlack					= 'black'
xtColorRed						= 'red'
xtColorBlue						= 'blue'
xtColorGreen					= 'green'

xtTimerModeOff					= 0
xtTimerModeSingle				= 1
xtTimerModeMultiple				= 2

xtDebugMode						= False


#
#  keys into the dictionary that is a command sent or received from XTension to update
#  the unit in XTension or from XTension to tell you to send the proper command to a device
#

#  COMMAND
xtKeyCommand 		 			= 'mcmd'
	# the command key will be included in every packet sent from XTension and must
	# be included in any command you send up to XTension. If you wish to only update
	# one of the values below that are normally sent along with a unit value change
	# event you can use the NoOp command and then include whatever else you need to.

xtCommandBlink 				= 'blink'
	# Received from XTension.
	# will include xtKeyAddress, xtKeyTag and xtKeyValue
	# BLINK received from the blink script command
	# in the xtKeyValue field will be whatever numerical or string value
	# is passed in the "rate" parameter. Of the verb
	# http://machomeautomation.com/doku.php/dictionary/unitcontrol/blink

xtCommandGetUnitProperty	= 'GetUnitProperty'
	# sent to XTension. 
	# requires xtKeyAddress, xtKeyTag and xtKeyName
	# include the name of the unit property you wish to retrieve 
	# in the xtKeyName field. Only text and numerical values can be
	# handled in this way. XTension will respond asynchronously with an
	# xtCommandSetUnitProperty command to your command handler with the 
	# data included.

xtCommandJSONRequest		= 'JSON'
	# sent to XTension.
	# requires xtKeyAddress, xtKeyTag
	# optional keys: xtKeyData, xtKeyJSONData, xtKeyUniqueID
	# a way to send JSON or other html form data to a special handler in a units ON script
	# I use this with the Vera support so that a user can get the JSON record
	# received for a unit in total and extract any extra data from it that they 
	# might wish to get out for devices I don't support automatically yet.
	# if the request includes GET or POST standard URL encoded values include them
	# as a string in the xtKeyData field. If there is JSON data include it in the
	# xtKeyJSONData field. The 2 datas will be turned into AppleScript records 
	# and passed to the "on jsonrequest( htmlData, jsonData)" handler in the units ON script
	# and there any action that they wish can be taken based on the data.
	# if you include a URL=/some/link/something parameter in the html form data then that
	# will also be passed to the script. If it ends in "/data" then the unit will respond with
	# all of the units internal settings and values converted to JSON and send a command with 
	# the same address and tag but with that keyData of JSON data back to your script.
	# The user in the script will also have the option of returning a value in reply to 
	# the request. If you wish to receive a reply from this call you must include a
	# unique ID for the request in the xtKeyUniqueID field. That can be any unique ID style
	# string. It will be included in the reply from XTension so that you can match it with
	# a request or an incoming connection. The reply command will have the same command key
	# and will include the same xtKeyUniqueID. The reply data will be included with the 
	# xtKeyData parameter. Additionally an HTML return code may be specified in xtKeyCode and
	# a content-type may be specified in the xtKeyContentType field. These responses are sent
	# asynchronously and no reply is guaranteed as the script may choose not to reply or
	# may suffer an error condition rather than returning anything.

xtCommandMessage			= 'MESS'
	# received from XTension
	# will contain xtKeyData
	# this comes from the Send Remote Message verb and will contain whatever text 
	# they included in the default parameter of the verb.
	# currently used to display messages on the front screens of thermostats that 
	# support such things.

xtCommandNewUnit			= 'NewU'
	# sent to XTension
	# requires xtkeyAddress, xtKeyTag and many others, see specific instructions for 
	# creating new units. If you can get the necessary units and their setup from your
	# device then you should create and send these commands whenever you are connected
	# if the units don't yet exist in XTension they will be created and added to whatever
	# list you specify saving the user from having to create them all manually and 
	# get them properly setup. If they already exist then they will be conformed to the
	# new settings to keep them updated. This has too many options to document here.

xtCommandNoOp				= 'NoOp'
	# sent to XTension
	# if you wish to send a key that rides along with another command but do not wish to
	# command a unit you can include the NoOp command.
	# for example if you wish to update the battery level for a device but
	# do not have a new value to send along with the change in battery status you 
	# can create a command with the xtKeyAddress and xtKeyTag and xtBatteryLevel
	# keys but with the NoOp command so that the unit will stil process the other included keys.
	# a command is always required in any packet. 


xtCommandOff				= 'OFF'
	# sent to XTension and received from XTension
	# requires xtKeyAddress and xtKeyTag
	# if received from XTension  you should take whatever action necessary to turn off the
	# addressed unit. If you receive a status update for a device for OFF you can create the
	# command and send it up to XTension. The addressed unit will then turn itself off
	# and run it's OFF script.


xtCommandOn					= 'ON'
	# sent to XTension and received from XTension
	# requires xtKeyAddress and xtKeyTag
	# may contain xtKeyValue
	# if received from XTension you should take whatever action necessary to turn on the
	# addressed unit. If the unit is configured as smart or simulated the command will
	# also include the xtKeyValue field which is the preset level the unit should return to.

xtCommandPing				= 'ping'
	# received from XTension 
	# if XTension hasn't heard from you in a while it may send a ping command just to make sure you're
	# still alive and able to communicate with your devices. If you receive a ping you should do whatever
	# is necessary to make sure you're still connected and operating properly and then respond
	# with an xtCommandPingResponse command. If you miss 2 consecutive ping requests the interface
	# will be forced to quit and restarted.

xtCommandPingResponse		= 'PSrp'
	# sent to XTension in response to the xtCommandPing request after you verify that
	# you're still able to communicate with your device. If you don't reply after 2 ping requests in a row
	# you will be force quit and restarted.

xtCommandQuery				= 'Qery'
	# received from XTension in response to the query verb for a unit
	# will contain the xtKeyAddress and xtKeyTag fields
	# You are being asked to perform a query out to the device being controlled for 
	# updating the value. When you receive a new value you should generate a normal
	# update command to XTension with the xtKeyUpdateOnly set to true.
	# queries are async, XTension will not wait for you to reply before more commands may be sent.

xtCommandReboot				= 'REBOOT'
	# received from XTension in response to the Reset Controller verb. If possible you should
	# ask the device or hub to reboot itself. You may also reset any default parameters that are
	# available in the device to reset it to some known state. Dont go so far as to reset the device
	# to factory defaults or anything such as that. This is a something is setup wrong or my device
	# may not be functioning correctly and needs a reboot command, not a factory reload command.

xtCommandRemoveXMLData		= 'rXML'
	# sent to XTension
	# requires xtKeyAddress, xtKeyTag and xtKeyName
	# removes the xtKeyName'd value from the units dictionary. You should use this only for 
	# values that you have added to the units key value pairs either via the dynamic interface
	# keys or with the xtCommandSetXMLData command. You must also make sure that you are using a
	# namespace reserved for your plugin. ALWAYS use PLUGINID_nameOfValue or some other method to
	# distinguish your keys from the built in keys in XTension. XTension reserves the use of all
	# key names that do not begin with a plugin tag and underscore.
	# no attempt is made to validate the name is not reserved for XTension before it is removed
	# so you may render the unit unusable if you delete an important key.

xtCommandSetDescription		= 'SetDesc'
	# sent to XTension
	# requires xtKeyAddress, xtKeyTag and xtKeyData
	# send this to XTension to set the "description" field of the unit to a string of your choosing.
	# the new description must be included in the xtKeyData field.
	# note that this will overwrite anything that the used has manually placed in this field.
	# Good for displaying text data about the value. For example a barameter sensor may wish 
	# to send a weather forecast to the description of the unit that holds the numerical value.

xtCommandSetUnitOfMeasure	= 'SetUnitOfMeasure'
	# sent to XTension
	# requires xtKeyAddress, xtKeyTag and xtKeyData
	# send this to change the value suffix of the unit to a new value. For example if the 
	# configuration of a temperature sensor has changed from F to C you may wish to send this
	# command to change the unit of measure to "C" so that will display in all interfaces after the value.
	# the new value will be included in the xtKeyData field.
	# this will overwrite any value that the user has placed in the value suffix field.

xtCommandSetUnitProperty 	= 'setUnitProperty'
	# sent to XTension and received from XTension in response to a xtCommandGetUnitProperty command
	# requires xtKeyAddress, xtKeyTag, xtKeyName, xtKeyData
	# sets a unit property given the name xtKeyName and the data xtKeydata both fields need to be
	# strings, or string representations of numbers. The key strings should be properly UTF8 encoded
	# this command is also returned to you with the selected data if you have sent an
	# xtCommandGetunitProperty command up to XTension. 

xtCommandSetValue			= 'SetValue'
	# sent to XTension and received from XTension
	# requires xtKeyAddress, xtKeyTag, xtKeyValue
	# if sent to XTension will update the addressed unit to the new numerical value and 
	# run the ON or OFF script.
	# if received from XTension the unit value has been changed and you should send whatever commands
	# to your connected device to bring the actual device to the same level.

xtCommandSetXMLData			= 'sXML'
	# sent to XTension
	# requires xtKeyAddress, xtKeyTag, xtKeyName, xtKeyValue
	# sent to XTension to save a string or numerical value into the root XML of a 
	# specific unit. This data will be available to you either with the 
	# xtCommandGetXMLData command or by getting the entirety of the 
	# units keyed data with all the setup info from the XTGetUnit command
	# you MUST use a prefix for your specific plugin for all data you store in the
	# units XML as it must share name space with built in and other data. Use
	# PLUGINNAME_valueName so that you don't overlap and cause a unit to become 
	# unusable. No validation is used to make sure you're not using a reserved key name.

xtCommandSimmPacket			= 'SimPacket'
	# received from XTension
	# will include xtKeyData
	# this is a debugging aid if you need to simulate a packet reception that is difficult
	# to re-create. You can use the Simulate Packet verb to send the string to yourself and use this
	# to insert it into the appropriate step in the parsing of the data for testing. This is
	# not used as any part of normal interface control, just for debugging and development.

xtCommandStop				= 'Stop'
	# received from XTension
	# will include xtKeyAddress and xtKeyTag
	# from the stop verb, currently used for window shade controls but can be used for anything sensible

xtCommandUnitDeleted		= 'UnitDeleted'
	# received from XTension
	# will include xtKeyAddress and xtKeyTag
	# if the user deletes a unit assigned to your interface you will receive this event to 
	# let you know and take whatever action you feel is appropriate. Generally it's not a good
	# idea to delete things in response to this, but stop any processes you might have outstanding
	# or regularly talking to the unit in question. 
	# There is no support for you sending a command up to XTension that will delete a users units. 
	# any disconnected units should be deleted by them manually after saving off any other data or
	# scripts associated with them.
	



# keys for sending data in XTension commands

xtKeyAddress 			 		= 'addr'
	# any command from a unit will have an address associated with it
	# global commands or commands not associated with a specific unit
	# will not include this. Look for the address only after you have 
	# validated the command, or make sure to use the .get method or
	# a try block
	
xtKeyTag 			 			= 'APfx'
	# any command from a unit will have an address tag. The main purpose
	# of this is so that units with overlapping addresses, because they are of different
	# types, can have a unique address in the database for finding which unit to 
	# work on. For example your device might support dimmer channels 1, 2, and 3 and 
	# create units with the addresses of 1, 2, and 3. Your device might also support
	# GPIO pins 1, 2, and 3. As long as the device types and their tags are different those
	# will still become unique address strings. The Tag is prefixed to the address before 
	# indexing. So the actual indexed address of the above examples might be "dimmer:1",
	# "dimmer:2", "dimmer:3", "gpio:1", "gpio:2", "gpio:3". All device types are required to
	# have a tag associated with them. It is also used for setting up the user interface
	# when editing the unit. It will always be included with any command from a unit
	# and must always be included in any command going to a unit.
	
	
xtKeyColorMode 					= 'ClMd'
	# In the event that a device supports both color mode and color temperature mode this
	# key will be present to tell you which mode is being selected or updated

xtKeyColorTemperature 		 	= 'CTmp'
	# if the device supports a color temperature selection this key will contain the
	# color temperature value.
	
xtKeyBatteryLevel 				= 'Batt'
	# when sending to XTension you can include a battery level reading if applicable.
	# including this will cause the battery icon in the flags to enable and display it
	# as a battery percentage. Valid values are from 0 to 100. Setting the value to 10 or 
	# less will cause the battery icon to show red and for the unit to be included in 
	# lists of low battery units.
	# for units that do not support a percentage just send 100 to cancel a no low batt flag or 
	# to show a green battery icon and 0 to display the low batt flag.
	
xtKeyCommError			 		= 'commError'
	# include a comm error number in any command to XTension and the unit will display
	# the alert icon next to it and run the interface error script for this unit when the
	# command is received. Send a 0 as the number to cancel the error. Error numbers are
	# not reserved and can be whatever makes sense for your own device and debugging.

xtKeyErrorMessage 				= 'ErrMessage'
	# in addition to an xtKeyCommError number you should include a message that describes the
	# error. This will be written to the log and also stored in the units properties so that
	# the error can be more easily made sense of.
	
xtKeyPowerLevel			 		= 'PwLv'
	# if your device supports a power level reading, for example how was the quality or power of  
	# the received signal, you can include it here and it will be available in the units ON/OFF
	# script by using the "single strength" verb.
	
xtKeyRate 						= 'Rate'
	# received from XTension, this is the value placed in the ramp rate parameter of the ON 
	# or dim/bri or set value verbs. If your device does not support this just ignore. 
	
xtKeyUpdateOnly	 				= 'UpDt'
	# sent to XTension. If you include this as a boolean true value in the command then
	# XTension will compare the new value you sent with the current value of the unit and
	# take no action if the values are the same. You should send the first values read on 
	# startup with this set always. That will conform the current database to the existing
	# state of things but will only update the units and run scripts if values are different
	# rather than make all units think they actually received an update when they really didnt.
	# for things like temp sensors you can send all values with this set and XTension will
	# just ignore ones that arent changed. No need to keep track of previous values and compare
	# then with each reception. 
	# The reception of this command does update the last message received date in the unit
	# even if the values are the same (NOTE: NOT the "last timestamp" or "time delta" value!)
	# this is used to verify that a device is actually still receiving data even if the value
	# hasn't changed. See the advanced tab of the Edit Unit dialog for more info on the 
	# alert if the unit becomes inactive for some reason.

xtKeyNoScript					= 'NScp'
	# causes the unit to update regardless of the value being the same or not, but
	# suppresses the running of the ON or OFF script
	
xtKeyOffLabel					= 'OfLb'
xtKeyOnLabel					= 'OnLb'
	# in any command you can include a new string for the ON or OFF label of the unit
	# for example, a thermostat unit would hold a numerical representation of the
	# state of the furnace, 0=off, 1=heat, 2=AC. But just showing 0, 1, 2 isn't very
	# informative so by setting the OFF label to "off" and including a new On label
	# that reflects the actual state in text along with the set value command it will
	# display everywhere what the unit is actually doing instead of just showing a number.
	# these can be specified when the unit is created and included with any other command
	# going to the unit to change it later. If included they will overwrite any options
	# the user might have added in the Edit Unit Dialog. 
	# they can also include advanced label syntax.
	
xtKeyTimestamp					= 'TStm'
	# sent to XTension. This parameter is optional for any command. If it is not included
	# XTension will use the current time when it processes the command as the new "last
	# timestamp value for the unit. If you are loading delayed data or processing older data
	# then you can include this and it will override the current date and be used as the
	# "last timestamp" value for the unit.
	# note that XTdb will not store values out of order. New records must be after the last record
	# that it added or the records will be ignored. They do not have to be current, they 
	# just have to be added in order chronologically. It is perfectly acceptable to batch
	# process a lot of value updates to get them into the database, as long as they are sent
	# in order they will be saved.
	
xtKeyValue						= 'Valu'
	# if the command requires a value it will be included with this key. An OFF command requires
	# no value so this will be absent. A dim, bright or new value command requires a value and it
	# will be included with this key. An ON command MAY include a value key if they unit is marked as
	# "smart" or "simulated" then the current preset value will be included in this value for an ON 
	# command as well. 


xtKeyUnitPropertyName			= 'UPN'
xtKeyUnitPropertyValue			= 'UPV'
	# Up to 10 unit properties can be sent in any command. They will be added to the unit properties
	# and all the new unit property events will be called. unit properties sent in this way can only
	# be string values, but they can be converted to numbers or anything else by the scripts that access
	# them. 
	# In this case you must alter the keys by adding an integer value from 0 to 0 after them, so instead of
	# including just the key you must include the key + "0" the key + "1" etc this makes the actual key length 
	# 4 characters "UPN0" and "UPV0" then "UPN1" and "UPN2" and so forth. They must be in order
	# as processing of them stops when the next number is not found. You must send both the propertyName
	# and PropertyValue keys. 

xtKeyColor						= "RGBW"
	# if color information is included in the ON or SetValue command then this will hold the RGB value
	# of the color you should display. It will already have the Value or Level of the light taken into
	# account so you do not have to do HSV calculations in the device if you dont want to. If you need
	# an option to receive the unaltered or root color value and apply the value or level yourself
	# please let me know.

xtKeyContentType				= 'CnTp'
	# received from XTension
	# used in the reply from the xtCommandJSONRequest when the user has generated some response
	# from the event. See the documentation for the JSONREquest handling
	
	
xtKeyCode						= 'Code'
	# received from XTension
	# used in the reply from the xtCommandJSONRequest when the user has generated some response
	# this will then be the HTTP return code. 200 for a success or might be some other number
	# for some error that the user wishes to respond.
	
xtKeyContentType				= 'CnTp'
	# received from XTension
	# used in the reply from the xtCommandJSONRequest when the user has generated some response
	# tells you the kind of data that they have generated.
	
xtKeyData						= 'DATA'
	# received from XTension and sent to XTension
	# used in many of the above commands to send the data associated with a command
	
xtKeyDimmable					= 'Dimm'
	# when sending the new unit command include this key set to True in order to mark the 
	# unit you're creating as dimmable
	
xtKeyJSONData					= 'JSdt'
	# used with the xtCommandJSONRequest if the request should include a JSON dictionary
	# it should be included as a string in this field. It will be parsed to an AppleScript dictionary
	# and passed to the units ON script.
	
xtKeyList						= 'List'
	# sent to XTension with the new unit command
	# if included the unit you create will be placed in a list with the name included in this
	# field. If the list doesn't exist it will be created.
	
xtKeyName						= 'Name'
	# used throughout the command set whenever a name is required. Also used when creating a new unit
	# for the default name to use when creating the unit. If the name is not unique XTension will add a
	# numerical value at the end to make it unique.
	
xtKeyReadOnly					= 'RdNl'
	# sent to XTension with the new unit command
	# if included with a True the unit will display with it's interface and type and other setup greyed out
	# so that the user cannot edit the basic unit properties, though display properties and scripts will always be 
	# editable. If it becomes necessary to edit these for any reason the used can hold the option key down when
	# opening the edit unit window and the fields will be enabled.
	
xtKeySuffix						= 'Sfix'
	# send to xtension with the new unit command
	# if included the string value in this field will fill in the unit of measure or display suffix field
	# of the new unit. For example a temperature unit might include a "F" in this value 
	# NOTE this is not the address suffix, just a display suffix.

xtKeyUniqueId					= 'unID'
	# included used in several commands back and forth from XTension where a uniqueID is needed
	
xtTrue							= 'true'
xtFalse							= 'false'
	# textual representation of true and false used in the commands to XTension



#
# Constants for accessing information in the Interface level communications dictionary
#

xtIntKeyRemoteAddress			= 'RemoteAddress'
	# if the connection type is for outgoing TCP then the remote address configured in the interface window will be stored here
xtIntKeyRemortPort				= 'RemotePort'
	# If the connection type is for outgoing TCP then the remote address port will be stored here
xtIntKeyName					= 'name'
	# The name given to your specific interface. This will already be pre-pended to any log lines you generate but it may be useful for other things.
xtIntKeyDescription				= 'Description'
	# the description of your interface plugin from the info.json file
xtIntKeyPortName				= 'PortName'
	# if NONE is selected then it will be
xtPortNameNone				= 'none'
	# if you're making an outgoing TCP connection then this value will be:
xtPortNameOutgoingTCP		= 'IP'
	#for accepting incoming connections the portName will be
xtPortNameIncomingTCP		= 'IPin'
	# if it isn't one of those then it will contain the unix name of the selected serial port
	
xtIntKeyFolderPath				= 'isf'
	# an escaped shell path to the folder that your plugin is running from
xtIntKeyUniqueID				= 'UniqueID'
	# a unique ID string unique to this instance of your plugin. Will be the same for every session for this config.



#
# Constants for access unit configuration information from the XTUnitById and XTUnitByAddress dictionary data
#

xtUnitKeyName					= 'Name'
	# the name of the unit
xtUnitKeyColorPreset			= 'cpreset'
	# if the unit has allowColor enabled then there will be 8 color presets
	# each key starts with this key and then appends a number going 0-7
	# the value will be a comma separated string of the R, G, and B values stored as the
	# color preset.
	
xtUnitKeyTag					= 'AddressPrefix'
	# the address prefix or tag as defined in the info.json file for this unit type
	# use this key to get it out of a unit info dictionary
	
xtUnitKeyIgnoreClicks			= 'IgnoreClicksInList'
	# if set to true the unit is not controllable by clicking on it in a list
	# but can still be controlled via the HUD control popups or scripts.
	# is an aide to not accidentally clicking on things that shouldn't be controlled

xtUnitKeyQuickControlType		= 'QuickControlType'
	# an integer value either 0 or 1. This is the saved off value of the last used
	# HUD popup window. If the value is 0 then the simple interface was last used and will
	# be opened again when the user next requests that window, if the value is 1 then the
	# advanced window was last used and will be shown next for this unit.
	
xtUnitKeyLastActivity			= 'LastActivity'
	# date as a string holding the last time the unit changed state, value or received
	# a command with the same value but without the only update if changed flag.
	
xtUnitKeyIgnoreOffs				= 'IgnoreOffs'
	# if true then the unit will not update it's last activity date when an OFF command
	# is received. Good for things like motion sensors where the off isn't as interesting
	# as the last ON received.
	
xtUnitKeySmart					= 'Smart'
	# the unit dimmable type is set to Smart. This means that XTension expects the unit
	# to return to the last preset level when an ON is sent rather than just go to 100%
	
xtUnitKeyPresetLevel			= 'PresetLevel'
	# if smart or simulated this is the value that the unit will return to when it's 
	# next sent an on command. 

xtUnitKeyReverseLogic			= 'ReverseLogic'
	# if set to true the unit will actually show the opposite of the last command received.

xtUnitKeyAddress				= 'Address'
	# the address of the unit as entered by the user into the Edit Unit dialog.

xtUnitKeyDeviceId				= 'Device'
	# the unique ID of the interface to which this unit is assigned. This will always be the
	# same as the interface unique id passed in the interface level configuration data 
	# since an interface can only talk directly to it's own units.
	
xtUnitKeyLastMessage			= 'lastMessage'
	# a date as a string holding the time the last message was received for this unit. Not
	# always the same as the last activity as the unit may be updating with the same value
	# and setting the only update if changed flag. This is the value used by the idle timeout
	# alert settings.
	
xtUnitKeyValue					= 'Value'
	# the stored value of the unit when the database was last saved. This value does not 
	# update with every change or command to the unit as the unit records are only refreshed
	# when the database is saved or when the unit is edited or saved.
	
xtUnitKeyDimmable				= 'Dimmable'
	# boolean, if the unit is setup to accept a value or just discrete on and off.

xtUnitKeyDeviceType				= 'DeviceType'
	# string description of the unit type. For plugin assigned units this
	# will always just say 'plugin'

xtUnitKeySimulated				= 'Simulated'
	# boolean if true then XTension will send the preset value instead of an ON command
	# as we are simulating a smart unit that remembered it's last value.


xtUnitKeyReceiveOnly			= 'ReceiveOnly'
	# boolean if true then the unit is set to only receive data and will not send a command
	# to the interface if the unit is controlled by the user or a script.
	
xtUnitKeyUniqueId				= 'UniqueID'
	# a unique ID string for this unit. 
	

xtUnitKeyBlocked				= 'Blocked'
	# boolean string, if true then the unit is blocked and will not respond to updates
	# nor send commands. A blocked unit that receives an update will update the database value
	# that it displays, but no scripts will be run.



# 
#
# private helper functions
# 
#

# some needed module variables

XTPluginInfo = {}
XTPluginConfig = {}
XTUnitList = []
XTUnitsById = {}
XTUnitsByAddress = {}

	# this is called by the interface for every unit that is assigned or saved to the
	# interface and just puts them into the global dictionary so that you can access
	# them by ID or address. There is a limitation in the framework that keeps me from
	# sending dictionaries as a python result object so we have to do it this way and
	# maintain the list in the python script.
def XT_SetUnit( unitData):
	global XTUnitsById
	global XTUnitsByAddress
	
	workAddress =  unitData[ 'AddressPrefix'] + unitData[ 'Address']
	workID = unitData['UniqueID']
	
	XTUnitsById[ workID] = unitData
	XTUnitsByAddress[ workAddress] = unitData
# 
# 	uncomment to see the entirety of the unit data available
# 	not everything in there will mean anything to your plugin though.
#	print( unitData)


#
# 	all info from the info.json file is available to the plugin in this global
# 	This is currently passed as a JSON string that you'll need to parse with json.load
# 	if you need anything out of this data file.
# 	the problem is that I don't yet have access to lists and embedded dictionaries
# 	inside a python dictionary. So In order to render all the structures properly I can 
# 	only convert it back to JSON and pass that as a string here.
#	this is not likely to be needed I don't think so I haven't set this up yet to anything
#	other than saving it as a string
#
def XT_SetPluginInfo( rawInfo):
	global XTPluginInfo
	XTPluginInfo = rawInfo

	
#
# all the configuration options set in the setup window and the custom interface
# controls are available in the XTPluginConfig global or with the XTGetPluginConfig call
#

def XT_SetPluginConfig( rawInfo):
	global XTPluginConfig
	XTPluginConfig = rawInfo
#	print( rawInfo)
	
def XTGetPluginConfig():
	global XTPluginConfig
	return XTPluginConfig
	

	
#
# some dictionaries may contain 'unicode' as a type instead of string
# due to limitations in the python API I can't get those out of a dictionary
# so any dictionary you create with strings from outside that might have unicode
# should be passed through this before you send it up to XTension
#
# 	this is only necessary if you're converting a JSON string to a python dictionary
#	this also only converts the top level objects and does not recurse through embedded
#	dictionaries or lists. After doing this though the keys will be of type string
#	and accessible. 
#	
#	this is not currently used anywhere anymore but I keep it here in case someone needs
#	access to something out of the XTPluginInfo JSON data.
#

def XTEnsureString( workDict):
	keys = workDict.keys()
	newDict = {}
	for workKey in keys:
		newDict[ str( workKey)] = str( workDict[ workKey])
	
	return newDict
